import type { Context } from 'hono';
export declare function returnResponse(ctx: Context, result: unknown): Response;
/**
 * SHA256 hash of the given values
 *
 * @param val one or more strings to hash
 * @returns hash string in hex format
 */
export declare function hash(...val: string[]): string;
/**
 * Safely stringify an object to JSON, handling circular references
 * @param obj - The object to stringify
 * @returns JSON string representation
 */
export declare function safeStringify(obj: unknown): string;
//# sourceMappingURL=_util.d.ts.map